/*
 *   Karn encryption
 *   Based on Phil Karn, sci.crypt, 13 Feb 1992
 *   Implemented by Alexander Myasnikov
 *
 *   Parameters: 768 bit key and 256 bit block
 *
 */

#include <mem.h>;

typedef unsigned long UINT4;

/* Initial values for MD5 Transform hash function */
static UINT4 ihash[4] = {
  0x67452301L, 0xefcdab89L, 0x98badcfeL, 0x10325476L
};

/* MD5 hash function */
extern void MD5Transform ();


UINT4 key1[12], key2[12];

void __stdcall __export
setup (UINT4 * key)
{
  bcopy (key, key1, 48);
  bcopy (key + 12, key2, 48);

}


void __stdcall __export
crypt (UINT4 * in)
{
  UINT4 out1[4], out2[4], in1[4], in2[4];

  int i;
  UINT4 buf[16];
  UINT4 hash[4];
  UINT4 temp[4];


  bcopy (in, in1, 16);
  bcopy (in + 4, in2, 16);

  bcopy (ihash, hash, sizeof (hash));
  bcopy (in1, buf, 16);
  bcopy (key1, buf + 4, 48);

  MD5Transform (hash, buf);

  for (i = 0; i < 4; ++i)
    temp[i] = buf[i] = in2[i] ^ hash[i];

  bcopy (ihash, hash, sizeof (hash));
  bcopy (key2, buf + 4, 48);

  MD5Transform (hash, buf);

  for (i = 0; i < 4; ++i)
    out2[i] = buf[i] = in1[i] ^ hash[i];

  bcopy (ihash, hash, sizeof (hash));
  bcopy (key1, buf + 4, 48);

  MD5Transform (hash, buf);

  for (i = 0; i < 4; ++i)
    out1[i] = temp[i] ^ hash[i];


  bcopy (out1, in, 16);
  bcopy (out2, in + 4, 16);


}




void __stdcall __export
decrypt (UINT4 * in)
{
  UINT4 out1[4], out2[4], in1[4], in2[4];

  int i;
  UINT4 buf[16];
  UINT4 hash[4];
  UINT4 temp[4];


  bcopy (in, in1, 16);
  bcopy (in + 4, in2, 16);

  for (i = 0; i < 4; ++i)
    buf[i] = in2[i];

  bcopy (ihash, hash, sizeof (hash));
  bcopy (key1, buf + 4, 48);
  MD5Transform (hash, buf);

  for (i = 0; i < 4; ++i)
    temp[i] = in1[i] ^ hash[i];

  for (i = 0; i < 4; ++i)
    buf[i] = temp[i];

  bcopy (ihash, hash, sizeof (hash));
  bcopy (key2, buf + 4, 48);
  MD5Transform (hash, buf);

  for (i = 0; i < 4; ++i)
    out1[i] = in2[i] ^ hash[i];


  bcopy (ihash, hash, sizeof (hash));
  bcopy (out1, buf, 16);
  bcopy (key1, buf + 4, 48);
  MD5Transform (hash, buf);

  for (i = 0; i < 4; ++i)
    out2[i] = temp[i] ^ hash[i];

  bcopy (out1, in, 16);
  bcopy (out2, in + 4, 16);


}






#include <string.h>


typedef unsigned char u8;
typedef unsigned long u32;

u32 __stdcall __export
getblocksize ()
{
  return 256;
}

u32 __stdcall __export
getkeysize ()
{
  return 768;
}

void __stdcall __export
getciphername (u8 * p)
{
  strcpy (p, "MD5-Karn");
}
